(function () {
	"use strict";
	
	var nsBoxManager = function(editor, url) {
		function createBox() {
			editor.undoManager.transact(function() {
				var classes = "ns-box";
				if ("true" == setting.borderRadius) classes += " ns-box-radius";
				
				editor.focus();
				editor.formatter.apply(
					"ns-box",
					{
						borderStyle: setting.borderStyle,
						borderColor: setting.borderColor,
						backgroundColor: setting.backgroundColor,
						classes: classes
					}
				);
				editor.nodeChanged();
			});
		}
		
		function removeBox() {
			var el = findNsBox();
			if (el) {
				editor.undoManager.transact(function() {
					editor.focus();
					editor.formatter.remove("ns-box", {value: null}, el);
					editor.nodeChanged();
				});
			}
		}
		
		function editBox(data) {
			var el = findNsBox(), style;
			if (el && data) {
				editor.undoManager.transact(function() {
					if (data.borderStyle) {
						editor.dom.setStyle(el, "border-style", data.borderStyle);
						style += "border-style: " + data.borderStyle + ";";
						editor.dom.setAttrib("mce-data-border-style", data.borderStyle);
						setting.borderStyle = data.borderStyle;
					}
					
					if (data.borderRadius) {
						if ("true" == data.borderRadius) {
							editor.dom.addClass(el, "ns-box-radius");
						} else {
							editor.dom.removeClass(el, "ns-box-radius");
						}
						setting.borderRadius = data.borderRadius;
					}
					
					if (data.borderColor) {
						editor.dom.setStyle(el, "border-color", data.borderColor);
						style += "border-color: " + data.borderColor + ";";
						editor.dom.setAttrib("mce-data-border-color", data.borderColor);
						setting.borderColor = data.borderColor;
					}
					
					if (data.backgroundColor) {
						editor.dom.setStyle(el, "background-color", data.backgroundColor);
						style += "background-color: " + data.backgroundColor + ";";
						editor.dom.setAttrib("mce-data-background-color", data.backgroundColor);
						setting.backgroundColor = data.backgroundColor;
					}
				});
			}
		}
		
		function createColorPickAction() {
			var colorPickerCallback = editor.settings.color_picker_callback;

			if (colorPickerCallback) {
				return function() {
					var self = this;

					colorPickerCallback.call(
						editor,
						function(value) {
							self.value(value).fire("change");
						},
						self.value()
					);
				};
			}
		}
		
		function renderEditBoxModal(dom) {
			var el = findNsBox(), borderStyle, borderRadius, borderColor, backgroundColor;
			
			if (el) {
				borderStyle = el.style.borderStyle || setting.borderStyle;
				borderRadius = el.classList.contains("ns-box-radius") ? "true" : "false";
				borderColor = editor.dom.toHex(el.style.borderColor) || editor.dom.toHex(setting.borderColor);
				backgroundColor = editor.dom.toHex(el.style.backgroundColor) || editor.dom.toHex(setting.backgroundColor);
				
				editor.windowManager.open( {
					title: "ボックスのスタイル",
					body: [
						{
							type: "listbox",
							name: "borderStyle",
							label: "線の種類",
							values: [
								{ text: "実線", value: "solid" },
								{ text: "点線", value: "dotted" },
								{ text: "無し", value: "none" }
							],
							value: borderStyle
						},
						{
							type: "listbox",
							name: "borderRadius",
							label: "角丸",
							values: [
								{ text: "有り", value: "true" },
								{ text: "無し", value: "false" }
							],
							value: borderRadius
						},
						{
							type: "colorbox",
							name: "borderColor",
							label: "ボーダー色",
							value: borderColor,
							onaction: createColorPickAction()
						},
						{
							type: "colorbox",
							name: "backgroundColor",
							label: "背景色",
							value: backgroundColor,
							onaction: createColorPickAction()
						}
					],
					onsubmit: function(e) {
						editBox(e.data);
					}
				});
			}
		}
		
		function findNsBox() {
			return findAncestor(editor.selection.getNode());
		}
		
		function findAncestor(el) {
			do {
				if (el.classList.contains("ns-box")) {
					return el;
				}
			} while ((el = el.parentElement));
			return false;
		}
		
		function handleDisabledState(ctrl, type) {
			function bindStateListener() {
				var el = findNsBox();
				ctrl.disabled(("add" == type && el) || ("edit" == type && !el));
			}
			
			if (editor.initialized) {
				bindStateListener();
			} else {
				editor.on("init", bindStateListener);
			}
			
			editor.on("SelectionChange change", bindStateListener);
		}
		
		function postAddRender() {
			handleDisabledState(this, "add");
		}
		
		function postEditRender() {
			handleDisabledState(this, "edit");
		}
		
		var setting = {
			"borderStyle": "solid",
			"borderRadius": "false",
			"borderColor": "#aaaaaa",
			"backgroundColor": "#fafafa",
		};
		
		editor.on("Init", function(e) {
			editor.formatter.register(
				"ns-box",
				{
					block: "div",
					wrapper: 1,
					styles: {
						borderStyle: "%borderStyle",
						borderColor: "%borderColor",
						backgroundColor: "%backgroundColor"
					},
					classes: "%classes",
					remove: "all"
				}
			);
		});
		
		editor.addButton("ns_box_button", {
			type: "menubutton",
			tooltip: "ボックス",
			icon: "icon mce-i-ns-box",
			menu: [
				{
					text: "ボックスを作成",
					onclick: createBox,
					onPostRender: postAddRender
				},
				{
					text: "ボックスを解除",
					onclick: removeBox,
					onPostRender: postEditRender
				},
				{
					text: "-"
				},
				{
					text: "ボックスのスタイル",
					onclick: renderEditBoxModal,
					onPostRender: postEditRender
				}
			]
		});
	};
	
	tinymce.PluginManager.add( "ns_box", nsBoxManager );
})();
